@@ -3,12 +3,13 @@ use crate::event::{Error, Event, Result, Status, UndoMgmt};
 use crate::map::{form_map, CharMap};
 use crate::searching::{Match, Searcher};
 use crate::utils::{
-    get_range, tab_boundaries_backward, tab_boundaries_forward, trim, width, Loc, Size,
+    get_range, modeline, tab_boundaries_backward, tab_boundaries_forward, trim, width, Loc, Size,
 };
 use ropey::Rope;
 use std::fs::File;
 use std::io::{BufReader, BufWriter};
 use std::ops::{Range, RangeBounds};
+use std::path::Path;
 
 /// A document info struct to store information about the file it represents
 #[derive(Clone, PartialEq, Eq, Debug)]
@@ -129,6 +130,24 @@ impl Document {
         Ok(this)
     }
 
+    /// Determine the file type of this file (represented by an extension)
+    #[allow(clippy::missing_panics_doc)]
+    #[must_use]
+    pub fn get_file_type(&self) -> Option<&str> {
+        let mut result = None;
+        // Try to use modeline first off
+        if let Some(first_line) = self.lines.first() {
+            result = modeline(first_line);
+        }
+        // If an extension is available, use that instead
+        if let Some(file_name) = &self.file_name {
+            if let Some(extension) = Path::new(file_name).extension() {
+                result = extension.to_str();
+            }
+        }
+        result
+    }
+
     /// Sets the tab display width measured in spaces, default being 4
     pub fn set_tab_width(&mut self, tab_width: usize) {
         self.tab_width = tab_width;
@@ -172,6 +191,7 @@ impl Document {
     /// Will return an error if the event was unable to be completed.
     pub fn exe(&mut self, ev: Event) -> Result<()> {
         if !self.info.read_only {
+            self.undo_mgmt.last_event = ev.clone();
             self.undo_mgmt.set_dirty();
             self.forth(ev)?;
         }
@@ -747,10 +767,6 @@ impl Document {
     /// Function to select to a specific x position
     pub fn select_to_x(&mut self, x: usize) {
         let line = self.line(self.loc().y).unwrap_or_default();
-        // If we're already at this x coordinate, just exit
-        if self.char_ptr == x {
-            return;
-        }
         // If the move position is out of bounds, move to the end of the line
         if line.chars().count() < x {
             let line = self.line(self.loc().y).unwrap_or_default();
@@ -790,6 +806,43 @@ impl Document {
         self.load_to(self.offset.y + self.size.h);
     }
 
+    /// Select a word at a location
+    pub fn select_word_at(&mut self, loc: &Loc) {
+        let y = loc.y;
+        let x = self.character_idx(loc);
+        let re = format!("(\t| {{{}}}|^|\\W| )", self.tab_width);
+        let start = if let Some(mut mtch) = self.prev_match(&re) {
+            let len = mtch.text.chars().count();
+            let same = mtch.loc.x + len == x;
+            if !same {
+                mtch.loc.x += len;
+            }
+            self.move_to(&mtch.loc);
+            if same && self.loc().x != 0 {
+                self.move_prev_word();
+            }
+            mtch.loc.x
+        } else {
+            0
+        };
+        let re = format!("(\t| {{{}}}|\\W|$|^ +| )", self.tab_width);
+        let end = if let Some(mtch) = self.next_match(&re, 0) {
+            mtch.loc.x
+        } else {
+            self.line(y).unwrap_or_default().chars().count()
+        };
+        self.move_to(&Loc { x: start, y });
+        self.select_to(&Loc { x: end, y });
+        self.old_cursor = self.loc().x;
+    }
+
+    /// Select a line at a location
+    pub fn select_line_at(&mut self, y: usize) {
+        let len = self.line(y).unwrap_or_default().chars().count();
+        self.move_to(&Loc { x: 0, y });
+        self.select_to(&Loc { x: len, y });
+    }
+
     /// Brings the cursor into the viewport so it can be seen
     pub fn bring_cursor_in_viewport(&mut self) {
         if self.offset.y > self.cursor.loc.y {
