@@ -1,5 +1,5 @@
 --[[
-Auto Indent v0.6
+Auto Indent v0.8
 
 Helps you when programming by guessing where indentation should go
 and then automatically applying these guesses as you program
@@ -149,8 +149,10 @@ function autoindent:disperse_block()
     local indenting_above = autoindent:causes_indent(editor.cursor.y - 1)
     local current_dedenting = autoindent:causes_dedent(editor.cursor.y)
     if indenting_above and current_dedenting then
+        local starting_indent = autoindent:get_indent(editor.cursor.y - 1)
         local old_cursor = editor.cursor
         editor:insert_line()
+        autoindent:set_indent(editor.cursor.y, starting_indent)
         editor:move_to(old_cursor.x, old_cursor.y)
     end
 end
@@ -168,10 +170,23 @@ event_mapping["enter"] = function()
     autoindent:disperse_block()
 end
 
-event_mapping["*"] = function()
-    -- Dedent where appropriate
-    if autoindent:causes_dedent(editor.cursor.y) then
-        local new_level = autoindent:get_indent(editor.cursor.y) - 1
-        autoindent:set_indent(editor.cursor.y, new_level)
+-- For each ascii characters and punctuation
+was_dedenting = false
+for i = 32, 126 do
+    local char = string.char(i)
+    -- ... excluding the global event binding
+    if char ~= "*" then
+        -- Keep track of whether the line was previously dedenting beforehand
+        event_mapping["before:" .. char] = function()
+            was_dedenting = autoindent:causes_dedent(editor.cursor.y)
+        end
+        -- Trigger dedent checking
+        event_mapping[char] = function()
+            -- Dedent where appropriate
+            if autoindent:causes_dedent(editor.cursor.y) and not was_dedenting then
+                local new_level = autoindent:get_indent(editor.cursor.y) - 1
+                autoindent:set_indent(editor.cursor.y, new_level)
+            end
+        end
     end
 end
