@@ -2,7 +2,7 @@ use crate::error::{OxError, Result};
 use crossterm::style::Color as CColor;
 use mlua::prelude::*;
 use std::collections::HashMap;
-use synoptic::{from_extension, Highlighter};
+use synoptic::Highlighter;
 
 use super::Color;
 
@@ -27,14 +27,6 @@ impl SyntaxHighlighting {
             )))
         }
     }
-
-    /// Get a highlighter given a file extension
-    pub fn get_highlighter(&self, ext: &str) -> Highlighter {
-        self.user_rules.get(ext).map_or_else(
-            || from_extension(ext, 4).unwrap_or_else(|| Highlighter::new(4)),
-            std::clone::Clone::clone,
-        )
-    }
 }
 
 impl LuaUserData for SyntaxHighlighting {
@@ -84,40 +76,36 @@ impl LuaUserData for SyntaxHighlighting {
         );
         methods.add_method_mut(
             "new",
-            |_, syntax_highlighting, (extensions, rules): (LuaTable, LuaTable)| {
-                // Make note of the highlighter
-                for ext_idx in 1..=(extensions.len()?) {
-                    // Create highlighter
-                    let mut highlighter = Highlighter::new(4);
-                    // Add rules one by one
-                    for rule_idx in 1..=(rules.len()?) {
-                        // Get rule
-                        let rule = rules.get::<i64, HashMap<String, String>>(rule_idx)?;
-                        // Find type of rule and attatch it to the highlighter
-                        match rule["kind"].as_str() {
-                            "keyword" => {
-                                highlighter.keyword(rule["name"].clone(), &rule["pattern"]);
-                            }
-                            "bounded" => highlighter.bounded(
-                                rule["name"].clone(),
-                                rule["start"].clone(),
-                                rule["end"].clone(),
-                                rule["escape"] == "true",
-                            ),
-                            "bounded_interpolation" => highlighter.bounded_interp(
-                                rule["name"].clone(),
-                                rule["start"].clone(),
-                                rule["end"].clone(),
-                                rule["i_start"].clone(),
-                                rule["i_end"].clone(),
-                                rule["escape"] == "true",
-                            ),
-                            _ => unreachable!(),
+            |_, syntax_highlighting, (name, rules): (String, LuaTable)| {
+                // Create highlighter
+                let mut highlighter = Highlighter::new(4);
+                // Add rules one by one
+                for rule_idx in 1..=(rules.len()?) {
+                    // Get rule
+                    let rule = rules.get::<i64, HashMap<String, String>>(rule_idx)?;
+                    // Find type of rule and attatch it to the highlighter
+                    match rule["kind"].as_str() {
+                        "keyword" => {
+                            highlighter.keyword(rule["name"].clone(), &rule["pattern"]);
                         }
+                        "bounded" => highlighter.bounded(
+                            rule["name"].clone(),
+                            rule["start"].clone(),
+                            rule["end"].clone(),
+                            rule["escape"] == "true",
+                        ),
+                        "bounded_interpolation" => highlighter.bounded_interp(
+                            rule["name"].clone(),
+                            rule["start"].clone(),
+                            rule["end"].clone(),
+                            rule["i_start"].clone(),
+                            rule["i_end"].clone(),
+                            rule["escape"] == "true",
+                        ),
+                        _ => unreachable!(),
                     }
-                    let ext = extensions.get::<i64, String>(ext_idx)?;
-                    syntax_highlighting.user_rules.insert(ext, highlighter);
                 }
+                syntax_highlighting.user_rules.insert(name, highlighter);
                 Ok(())
             },
         );
