@@ -7,8 +7,10 @@ use super::Editor;
 impl Editor {
     /// Execute an edit event
     pub fn exe(&mut self, ev: Event) -> Result<()> {
+        if !self.doc().undo_mgmt.last_event.same_type(&ev) && !self.plugin_active {
+            self.doc_mut().commit();
+        }
         self.doc_mut().exe(ev)?;
-        // TODO: Check for change in event type and commit to undo/redo stack if present
         Ok(())
     }
 
@@ -26,19 +28,14 @@ impl Editor {
         } else {
             let loc = self.doc().char_loc();
             self.exe(Event::Insert(loc, ch.to_string()))?;
-            self.highlighter[self.ptr].edit(loc.y, &self.doc[self.ptr].lines[loc.y]);
-        }
-        // Commit to event stack (for undo/redo if the character is a space)
-        if ch == ' ' {
-            self.doc_mut().commit();
+            let file = &mut self.files[self.ptr];
+            file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
         }
         Ok(())
     }
 
     /// Handle the return key
     pub fn enter(&mut self) -> Result<()> {
-        // When the return key is pressed, we want to commit to the undo/redo stack
-        self.doc_mut().commit();
         // Perform the changes
         if self.doc().loc().y == self.doc().len_lines() {
             // Enter pressed on the empty line at the bottom of the document
@@ -47,10 +44,11 @@ impl Editor {
             // Enter pressed in the start, middle or end of the line
             let loc = self.doc().char_loc();
             self.exe(Event::SplitDown(loc))?;
-            let line = &self.doc[self.ptr].lines[loc.y + 1];
-            self.highlighter[self.ptr].insert_line(loc.y + 1, line);
-            let line = &self.doc[self.ptr].lines[loc.y];
-            self.highlighter[self.ptr].edit(loc.y, line);
+            let file = &mut self.files[self.ptr];
+            let line = &file.doc.lines[loc.y + 1];
+            file.highlighter.insert_line(loc.y + 1, line);
+            let line = &file.doc.lines[loc.y];
+            file.highlighter.edit(loc.y, line);
         }
         Ok(())
     }
@@ -58,10 +56,10 @@ impl Editor {
     /// Handle the backspace key
     pub fn backspace(&mut self) -> Result<()> {
         if !self.doc().is_selection_empty() {
+            // Removing a selection is significant and worth an undo commit
             self.doc_mut().commit();
             self.doc_mut().undo_mgmt.set_dirty();
             self.doc_mut().remove_selection();
-            // Removing a selection is significant and worth an undo commit
             self.reload_highlight();
             return Ok(());
         }
@@ -74,10 +72,12 @@ impl Editor {
             let mut loc = self.doc().char_loc();
             self.highlighter().remove_line(loc.y);
             loc.y = loc.y.saturating_sub(1);
-            loc.x = self.doc().line(loc.y).unwrap().chars().count();
+            let file = &mut self.files[self.ptr];
+            loc.x = file.doc.line(loc.y).unwrap().chars().count();
             self.exe(Event::SpliceUp(loc))?;
-            let line = &self.doc[self.ptr].lines[loc.y];
-            self.highlighter[self.ptr].edit(loc.y, line);
+            let file = &mut self.files[self.ptr];
+            let line = &file.doc.lines[loc.y];
+            file.highlighter.edit(loc.y, line);
         } else if !(c == 0 && on_first_line) {
             // Backspace was pressed in the middle of the line, delete the character
             c = c.saturating_sub(1);
@@ -88,7 +88,8 @@ impl Editor {
                         y: self.doc().loc().y,
                     };
                     self.exe(Event::Delete(loc, ch.to_string()))?;
-                    self.highlighter[self.ptr].edit(loc.y, &self.doc[self.ptr].lines[loc.y]);
+                    let file = &mut self.files[self.ptr];
+                    file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
                 }
             }
         }
@@ -105,7 +106,8 @@ impl Editor {
                     y: self.doc().loc().y,
                 };
                 self.exe(Event::Delete(loc, ch.to_string()))?;
-                self.highlighter[self.ptr].edit(loc.y, &self.doc[self.ptr].lines[loc.y]);
+                let file = &mut self.files[self.ptr];
+                file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
             }
         }
         Ok(())
@@ -122,8 +124,6 @@ impl Editor {
 
     /// Delete the current line
     pub fn delete_line(&mut self) -> Result<()> {
-        // Commit events to event manager (for undo / redo)
-        self.doc_mut().commit();
         // Delete the line
         if self.doc().loc().y < self.doc().len_lines() {
             let y = self.doc().loc().y;
