@@ -1,7 +1,9 @@
 use crate::config::{Config, Indentation};
 use crate::error::{OxError, Result};
 use crate::ui::{size, Feedback, Terminal};
-use crossterm::event::{Event as CEvent, KeyCode as KCode, KeyModifiers as KMod, MouseEventKind};
+use crossterm::event::{
+    Event as CEvent, KeyCode as KCode, KeyModifiers as KMod, MouseEvent, MouseEventKind,
+};
 use kaolinite::event::Error as KError;
 use kaolinite::Document;
 use mlua::{Error as LuaError, Lua};
@@ -10,12 +12,18 @@ use std::time::Instant;
 use synoptic::Highlighter;
 
 mod cursor;
+mod documents;
 mod editing;
+mod filetypes;
 mod interface;
 mod mouse;
 mod scanning;
 
+pub use documents::FileContainer;
+pub use filetypes::{FileType, FileTypes};
+
 /// For managing all editing and rendering of cactus
+#[allow(clippy::struct_excessive_bools)]
 pub struct Editor {
     /// Interface for writing to the terminal
     pub terminal: Terminal,
@@ -24,9 +32,7 @@ pub struct Editor {
     /// Configuration information for the editor
     pub config: Config,
     /// Storage of all the documents opened in the editor
-    pub doc: Vec<Document>,
-    /// Syntax highlighting integration
-    pub highlighter: Vec<Highlighter>,
+    pub files: Vec<FileContainer>,
     /// Pointer to the document that is currently being edited
     pub ptr: usize,
     /// true if the editor is still running, false otherwise
@@ -43,6 +49,10 @@ pub struct Editor {
     pub push_down: usize,
     /// Used to cache the location of the configuration file
     pub config_path: String,
+    /// Flag to determine whether or not the editor is under control by a plug-in
+    pub plugin_active: bool,
+    /// Stores the last click the user made (in order to detect double-click)
+    pub last_click: Option<(Instant, MouseEvent)>,
 }
 
 impl Editor {
@@ -50,19 +60,20 @@ impl Editor {
     pub fn new(lua: &Lua) -> Result<Self> {
         let config = Config::new(lua)?;
         Ok(Self {
-            doc: vec![],
+            files: vec![],
             ptr: 0,
             terminal: Terminal::new(config.terminal.clone()),
             config,
             active: true,
             greet: false,
             needs_rerender: true,
-            highlighter: vec![],
             feedback: Feedback::None,
             command: None,
             last_active: Instant::now(),
             push_down: 1,
             config_path: "~/.oxrc".to_string(),
+            plugin_active: false,
+            last_click: None,
         })
     }
 
@@ -83,23 +94,26 @@ impl Editor {
         // Update in the syntax highlighter
         let mut highlighter = Highlighter::new(4);
         highlighter.run(&doc.lines);
-        self.highlighter.push(highlighter);
         // Add document to documents
-        self.doc.push(doc);
+        self.files.push(FileContainer {
+            highlighter,
+            file_type: Some(FileType::default()),
+            doc,
+        });
         Ok(())
     }
 
     /// Create a new document and move to it
     pub fn new_document(&mut self) -> Result<()> {
         self.blank()?;
-        self.ptr = self.doc.len().saturating_sub(1);
+        self.ptr = self.files.len().saturating_sub(1);
         Ok(())
     }
 
     /// Create a blank document if none are already opened
     pub fn new_if_empty(&mut self) -> Result<()> {
         // If no documents were provided, create a new empty document
-        if self.doc.is_empty() {
+        if self.files.is_empty() {
             self.blank()?;
             self.greet = self.config.greeting_message.borrow().enabled;
         }
@@ -111,32 +125,32 @@ impl Editor {
         let mut size = size()?;
         size.h = size.h.saturating_sub(1 + self.push_down);
         let mut doc = Document::open(size, file_name)?;
-        doc.set_tab_width(self.config.document.borrow().tab_width);
-        // Load all the lines within viewport into the document
+        // Collect various data from the document
+        let tab_width = self.config.document.borrow().tab_width;
+        let file_type = self.config.document.borrow().file_types.identify(&mut doc);
+        // Set up the document
+        doc.set_tab_width(tab_width);
         doc.load_to(size.h);
-        // Update in the syntax highlighter
-        let mut ext = file_name.split('.').last().unwrap_or("");
-        if ext == "oxrc" {
-            ext = "lua";
-        }
-        let mut highlighter = self
-            .config
-            .syntax_highlighting
-            .borrow()
-            .get_highlighter(ext);
-        highlighter.run(&doc.lines);
-        self.highlighter.push(highlighter);
         doc.undo_mgmt.saved();
-        // Add document to documents
-        self.doc.push(doc);
+        // Update in the syntax highlighter
+        let mut highlighter = file_type.as_ref().map_or(Highlighter::new(tab_width), |t| {
+            t.get_highlighter(&self.config, tab_width)
+        });
+        highlighter.run(&doc.lines);
+        // Add in the file
+        self.files.push(FileContainer {
+            doc,
+            highlighter,
+            file_type,
+        });
         Ok(())
     }
 
     /// Function to ask the user for a file to open
     pub fn open_document(&mut self) -> Result<()> {
-        let path = self.prompt("File to open")?;
+        let path = self.path_prompt()?;
         self.open(&path)?;
-        self.ptr = self.doc.len().saturating_sub(1);
+        self.ptr = self.files.len().saturating_sub(1);
         Ok(())
     }
 
@@ -145,21 +159,28 @@ impl Editor {
         let file = self.open(&file_name);
         if let Err(OxError::Kaolinite(KError::Io(ref os))) = file {
             if os.kind() == ErrorKind::NotFound {
+                // Create a new document if not found
                 self.blank()?;
-                let binding = file_name.clone();
-                let ext = binding.split('.').last().unwrap_or("");
-                self.doc.last_mut().unwrap().file_name = Some(file_name);
-                self.doc.last_mut().unwrap().info.modified = true;
-                let highlighter = self
+                let file = self.files.last_mut().unwrap();
+                file.doc.file_name = Some(file_name);
+                // Work out information for the document
+                let tab_width = self.config.document.borrow().tab_width;
+                let file_type = self
                     .config
-                    .syntax_highlighting
+                    .document
                     .borrow()
-                    .get_highlighter(ext);
-                *self.highlighter.last_mut().unwrap() = highlighter;
-                self.highlighter
-                    .last_mut()
-                    .unwrap()
-                    .run(&self.doc.last_mut().unwrap().lines);
+                    .file_types
+                    .identify(&mut file.doc);
+                // Set up the document
+                file.doc.info.modified = true;
+                file.doc.set_tab_width(tab_width);
+                // Attach the correct highlighter
+                let highlighter = file_type.clone().map_or(Highlighter::new(tab_width), |t| {
+                    t.get_highlighter(&self.config, tab_width)
+                });
+                file.highlighter = highlighter;
+                file.highlighter.run(&file.doc.lines);
+                file.file_type = file_type;
                 Ok(())
             } else {
                 file
@@ -185,14 +206,23 @@ impl Editor {
         let file_name = self.prompt("Save as")?;
         self.doc_mut().save_as(&file_name)?;
         if self.doc().file_name.is_none() {
-            let ext = file_name.split('.').last().unwrap_or("");
-            self.highlighter[self.ptr] = self
+            // Get information about the document
+            let file = self.files.last_mut().unwrap();
+            let tab_width = self.config.document.borrow().tab_width;
+            let file_type = self
                 .config
-                .syntax_highlighting
+                .document
                 .borrow()
-                .get_highlighter(ext);
-            self.doc_mut().file_name = Some(file_name.clone());
-            self.doc_mut().info.modified = false;
+                .file_types
+                .identify(&mut file.doc);
+            // Reattach an appropriate highlighter
+            let highlighter = file_type.map_or(Highlighter::new(tab_width), |t| {
+                t.get_highlighter(&self.config, tab_width)
+            });
+            file.highlighter = highlighter;
+            file.highlighter.run(&file.doc.lines);
+            file.doc.file_name = Some(file_name.clone());
+            file.doc.info.modified = false;
         }
         // Commit events to event manager (for undo / redo)
         self.doc_mut().commit();
@@ -203,10 +233,10 @@ impl Editor {
 
     /// Save all the open documents to the disk
     pub fn save_all(&mut self) -> Result<()> {
-        for doc in &mut self.doc {
-            doc.save()?;
+        for file in &mut self.files {
+            file.doc.save()?;
             // Commit events to event manager (for undo / redo)
-            doc.commit();
+            file.doc.commit();
         }
         self.feedback = Feedback::Info("Saved all documents".to_string());
         Ok(())
@@ -214,23 +244,22 @@ impl Editor {
 
     /// Quit the editor
     pub fn quit(&mut self) -> Result<()> {
-        self.active = !self.doc.is_empty();
+        self.active = !self.files.is_empty();
         // If there are still documents open, only close the requested document
         if self.active {
             let msg = "This document isn't saved, press Ctrl + Q to force quit or Esc to cancel";
             if !self.doc().info.modified || self.confirm(msg)? {
-                self.doc.remove(self.ptr);
-                self.highlighter.remove(self.ptr);
+                self.files.remove(self.ptr);
                 self.prev();
             }
         }
-        self.active = !self.doc.is_empty();
+        self.active = !self.files.is_empty();
         Ok(())
     }
 
     /// Move to the next document opened in the editor
     pub fn next(&mut self) {
-        if self.ptr + 1 < self.doc.len() {
+        if self.ptr + 1 < self.files.len() {
             self.ptr += 1;
         }
     }
@@ -244,22 +273,22 @@ impl Editor {
 
     /// Returns a document at a certain index
     pub fn get_doc(&mut self, idx: usize) -> &mut Document {
-        self.doc.get_mut(idx).unwrap()
+        &mut self.files.get_mut(idx).unwrap().doc
     }
 
     /// Gets a reference to the current document
     pub fn doc(&self) -> &Document {
-        self.doc.get(self.ptr).unwrap()
+        &self.files.get(self.ptr).unwrap().doc
     }
 
     /// Gets a mutable reference to the current document
     pub fn doc_mut(&mut self) -> &mut Document {
-        self.doc.get_mut(self.ptr).unwrap()
+        &mut self.files.get_mut(self.ptr).unwrap().doc
     }
 
     /// Gets the number of documents currently open
     pub fn doc_len(&mut self) -> usize {
-        self.doc.len()
+        self.files.len()
     }
 
     /// Load the configuration values
