@@ -24,6 +24,7 @@ use std::rc::Rc;
 use std::result::Result as RResult;
 use ui::Feedback;
 
+/// Entry point - grabs command line arguments and runs the editor
 fn main() {
     // Interact with user to find out what they want to do
     let cli = CommandLineInterface::new();
@@ -37,6 +38,8 @@ fn main() {
     }
 }
 
+/// Run the editor
+#[allow(clippy::too_many_lines)]
 fn run(cli: &CommandLineInterface) -> Result<()> {
     // Create lua interpreter
     let lua = Lua::new();
@@ -51,6 +54,9 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
     let editor = Rc::new(RefCell::new(editor));
     lua.globals().set("editor", editor.clone())?;
 
+    // Inject the networking library for plug-ins to use
+    handle_lua_error(&editor, "", lua.load(PLUGIN_NETWORKING).exec());
+
     // Load config and initialise
     lua.load(PLUGIN_BOOTSTRAP).exec()?;
     let result = editor.borrow_mut().load_config(&cli.config_path, &lua);
@@ -59,9 +65,6 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
         handle_lua_error(&editor, "configuration", Err(err));
     };
 
-    // Inject the networking library for plug-ins to use
-    handle_lua_error(&editor, "", lua.load(PLUGIN_NETWORKING).exec());
-
     // Open files user has asked to open
     for (c, file) in cli.to_open.iter().enumerate() {
         // Open the file
@@ -71,15 +74,22 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
             editor.borrow_mut().get_doc(c).info.read_only = true;
         }
         // Set highlighter if applicable
-        if let Some(ref ext) = cli.file_type {
-            let mut highlighter = editor
-                .borrow()
+        if let Some(ref file_type) = cli.file_type {
+            let tab_width = editor.borrow().config.document.borrow().tab_width;
+            let file_type = editor
+                .borrow_mut()
                 .config
-                .syntax_highlighting
+                .document
                 .borrow()
-                .get_highlighter(ext);
+                .file_types
+                .get_name(file_type)
+                .unwrap_or_default();
+            let mut highlighter = file_type.get_highlighter(&editor.borrow().config, tab_width);
             highlighter.run(&editor.borrow_mut().get_doc(c).lines);
-            *editor.borrow_mut().get_highlighter(c) = highlighter;
+            let mut editor = editor.borrow_mut();
+            let file = editor.files.get_mut(c).unwrap();
+            file.highlighter = highlighter;
+            file.file_type = Some(file_type);
         }
     }
 
@@ -96,7 +106,7 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
         let hl = holder.get_highlighter(this_doc);
         hl.run(&lines);
         if cli.flags.read_only {
-            editor.borrow_mut().get_doc(this_doc).info.read_only = true;
+            holder.get_doc(this_doc).info.read_only = true;
         }
     }
 
@@ -170,11 +180,11 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
         editor.borrow_mut().command = None;
     }
 
-    // Exit
     editor.borrow_mut().terminal.end()?;
     Ok(())
 }
 
+/// Handle a lua error, showing the user an informative error
 fn handle_lua_error(editor: &Rc<RefCell<Editor>>, key_str: &str, error: RResult<(), mlua::Error>) {
     match error {
         // All good
@@ -245,7 +255,7 @@ fn handle_lua_error(editor: &Rc<RefCell<Editor>>, key_str: &str, error: RResult<
     }
 }
 
-// Run a command in the editor
+/// Run a command in the editor
 fn run_editor_command(editor: &Rc<RefCell<Editor>>, cmd: &str, lua: &Lua) {
     let cmd = cmd.replace('\'', "\\'").to_string();
     if let [subcmd, arguments @ ..] = cmd.split(' ').collect::<Vec<&str>>().as_slice() {
